
function EIDOLON=make_eidolon(image_matrix,REACH,COHERENCE,FIX_STD,DISARRAY_MASK);

%USAGE:   EIDOLON=make_eidolon(image_matrix,REACH,COHERENCE,FIX_STD,DISARRAY_MASK);
%
%
% Matlab implementation of the Eidolon Factory (see...
%
% Written by Matteo Valsecchi, Jan Koenderink and Andrea Van Doorn in
% October 2015 based on the PROCESSING version
%
% INPUTS:
%
% image_matrix: Can be either grayscale or color image
% REACH: Represent the maximum displacement (in pixel) by which each scale can be disarrayed
%     Parameter can be either
%     1) Scalar: the same value of disarray reach is applied to all scales
%     2) 2-element vector: the first reach value is appied to the finest scale, the second to the coarsest scale and the remaining scales have linearly spaced values
%     3) Multiple element vector: the reach values are applied in order to the scales (from the finer to coarser). WARNING this fixes the number of scales, i.e. if you use a 6 element vector, 6 scales will be computed.
% COHERENCE: Scalar comprised between 0 (disarray is completely incoherent between scales) and 1 (the same noise field is used to disarray all scales);
% FIX_STD(optional) imposes the kernels to be used (number and std). Must
%   be increasing values in pixels. WARNING if FIX_STD is defined, REACH
%   must be either a scalar, a 2-element vector OR have THE SAME SIZE AS
%   FIX_STD. WARNING if you use very large STDs the process can slow down
%   considerably
% DISARRAY_MASK(optional) Masks disarray fields. Must be a matrix same size
% as image_matrix, with values between 0 and 1.The values are multiplied
% with the disarray fields, so that 1 means full disarray and 0 no
% disarray.
%
% OUTPUT:
%
% EIDOLON: Has the same format as the image_matrix (color or grayscale image);



image_matrix=double(image_matrix);

IMSIZE=size(image_matrix);

sizeim=length(IMSIZE);
if ~any(sizeim==[2 3]);
    error('Image Must Be NxM or NxMx3');
end
if any(IMSIZE(1:2)<2)
    error('Image Must Be NxM or NxMx3 with both N and M lager than 1');
end



%%%% Define a few parameters


% color or grayscale image
if sizeim==3;
    NLAYERS=3;
else
    NLAYERS=1;
end



if nargin<2
    warning('Using Default Reach Value (image size/100)');
    REACH=max(IMSIZE/100);
end

isreachvector=0;
if prod(size(REACH))==1
    MIN_REACH=REACH;
    MAX_REACH=REACH;
elseif prod(size(REACH))==2
    MIN_REACH=REACH(1);
    MAX_REACH=REACH(2);
elseif (isvector(REACH))
    isreachvector=1;
    warning('You introduced a customized reach vector (ok if you know what you are doing)')
else
    error('REACH must be a scalar, a 2 element vector for linearly increasing or decreasing reach or a multiple element vector (for customized reach profile)')
end

isfixstd=0;
if ((nargin>=4)&(~isempty(FIX_STD)))
    isfixstd=1;
    
    if length(REACH)>2
        if length(REACH)~=length(FIX_STD)
            error('If FIX_STD is defined and REACH is more than 2 elements, they must have the same length.')
        end
    end
    if ~all(diff(FIX_STD)>0)
        error('FIX_STD must be monotonic increasing');
    end
end

mask_disarray_field=0;
if ((nargin>=5)&(~isempty(DISARRAY_MASK)))
    if ((any(DISARRAY_MASK(:))>1) | (any(DISARRAY_MASK(:))<0))
        error('Disarray mask values need to be between 0 and 1');
    end
    if length(size(DISARRAY_MASK))>2
        error('DISARRAY_MASK should be NxM with as many rows and columns as image_matrix');
    elseif any(size(DISARRAY_MASK)~=IMSIZE(1:2))
        error('DISARRAY_MASK should be NxM with as many rows and columns as image_matrix');
    end
    
    mask_disarray_field=1;
end

% number of scales and std of Gaussian kernel for each scale based on image size
N_of_Scales=1;
PixScaleSizes=1;
tps=1;

if isfixstd
    PixScaleSizes=FIX_STD;
    N_of_Scales=length(FIX_STD);
elseif ~isreachvector;
    while tps<(max(IMSIZE/4)) % std from 1 to 1/4 of image size (on larger dimension)
        tps=tps*sqrt(2);
        N_of_Scales=N_of_Scales+1;
        PixScaleSizes(N_of_Scales)=tps;
    end
else
    for N_of_Scales=2:length(REACH)
        tps=tps*sqrt(2);
        PixScaleSizes(N_of_Scales)=tps;
    end
end


if nargin<3
    warning('Using Default Coherence Value (.5)');
    COHERENCE=.5;
end

if ((COHERENCE>1) | (COHERENCE<0))
    error('Coherence must be between 0 and 1');
end


%%%% Start Actual Computation

disp('Compute image spectrum');
tic
for nlay=1:NLAYERS
    imagespect{nlay}=fft2(squeeze(image_matrix(:,:,nlay)));
end
disp(['Done ',num2str( toc),' s']);



disp('Setup Gaussian Kernels');
tic
GAUSS_KRNL_SPECT=Setup_Gaussian_Kernels(IMSIZE,PixScaleSizes);
disp(['Done ',num2str( toc),' s']);


if ~isreachvector;
    % Define Reach for each scale
    REACH_SCALE=linspace(MIN_REACH,MAX_REACH,length(GAUSS_KRNL_SPECT));
else
    REACH_SCALE=REACH;
end


disp('Decompose image');
tic
Diff_scaleim=Decompose_Image(imagespect,GAUSS_KRNL_SPECT);
disp(['Done ',num2str( toc),' s']);


disp('Setup disarray');
tic
RANDF_scale=Setup_disarray(GAUSS_KRNL_SPECT,COHERENCE,REACH_SCALE);
disp(['Done ',num2str( toc),' s']);

if mask_disarray_field
    disp('Mask Disarray Fields');
        tic
    for nscale=1:length(RANDF_scale)
        for coorxy=1:2
            RANDF_scale{nscale}(:,:,coorxy)=RANDF_scale{nscale}(:,:,coorxy).*DISARRAY_MASK;
        end
    end

    disp(['Done ',num2str( toc),' s']);
end


disp('Apply Disarray');
tic
EIDOLON_scale=Apply_Disarray(RANDF_scale,Diff_scaleim);
disp(['Done ',num2str( toc),' s']);

disp('Assemble Eidolon');
tic
EIDOLON=Assemble_Eidolon(EIDOLON_scale);
disp(['Done ',num2str( toc),' s']);



